package edu.uoc.android.memoriesplanner;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;

import android.view.MenuItem;

/**
 * An activity representing a single Event detail screen. This
 * activity is only used on narrow width devices. On tablet-size devices,
 * item details are presented side-by-side with a list of items.
 */
public class EventDetailActivity extends AppCompatActivity {

    //ONCREATE METHOD
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_event_detail);

        //Top toolbar
        Toolbar toolbar = (Toolbar) findViewById(R.id.detail_toolbar);
        setSupportActionBar(toolbar);

        // Show the Back button in the action bar.
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        if (savedInstanceState == null) {
            // Create the detail fragment and add it to the activity
            // using a fragment transaction.
            Bundle arguments = new Bundle();
            arguments.putString(EventDetailFragment.ARG_EVENT_TITLE,
                    getIntent().getStringExtra(EventDetailFragment.ARG_EVENT_TITLE));
            EventDetailFragment fragment = new EventDetailFragment();
            fragment.setArguments(arguments);
            getSupportFragmentManager().beginTransaction()
                    .add(R.id.event_detail_container, fragment)
                    .commit();
        }
    }

    //ONCREATEOPTIONSMENU METHOD.
    //Set the top bar menu
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    //ONOPTIONSITEMSELECTED METHOD
    //Navigation between activities using the toolbar menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        Intent navigation_intent;
        switch (item.getItemId()) {
            case R.id.future_events:
                navigation_intent = new Intent(this, EventFutureListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.past_events:
                navigation_intent = new Intent(this, EventPastListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.calendar:
                navigation_intent = new Intent(this, CalendarActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case android.R.id.home:
                this.finish();
                break;
            default:
                break;
        }
        return true;
    }
}
