package edu.uoc.android.memoriesplanner;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import android.support.v7.widget.Toolbar;

import com.orm.SugarContext;
import com.squareup.picasso.Picasso;

import org.json.JSONArray;
import org.json.JSONException;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import java.util.Calendar;

import java.util.List;
import java.util.Locale;

import edu.uoc.android.memoriesplanner.adapter.ImageListAdapter;
import edu.uoc.android.memoriesplanner.model.EventContent;

import static android.app.Activity.RESULT_OK;

/**
 * A fragment representing a single Event detail screen.
 * This fragment is either contained in a {@link EventFutureListActivity}
 * in two-pane mode (on tablets) or a {@link EventDetailActivity}
 * on handsets.
 */
public class EventDetailFragment extends Fragment {
    //Fragment argument representing the event title
    public static final String ARG_EVENT_TITLE = "event_title";

    private EventContent.EventItem mItem;           //The event content this fragment is presenting

    private ImageListAdapter adapter;               //Adapter of the photos album
    private List<String> imageList;                 //List of album photos paths

    private EditText editTitle;                     //Event title input
    private EditText editPlace;                     //Event place input
    private EditText editDescription;               //Event description input
    private DatePicker datePickerInitial;           //Event initial date picker
    private DatePicker datePickerFinal;             //Event final date picker
    private ImageView photo;                        //Event main photo
    private ImageButton photoDeleteButton;          //Event main photo delete button
    private TextView photoText;                     //"Add photo" hint text of the event main photo
    private ImageView addPhotoAlbum;                //Button to add photo to the album
    private ImageButton saveButton;                 //Save event changes button
    private ImageButton deleteButton;               //delete event button

    //Date format for the dates stored in the database
    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.ENGLISH);

    /*Image loading*/
    private static int PICK_IMAGE_REQUEST = 1;                  //Pick main photo code
    private String mainImagePath = "";                          //Main photo path
    private static int PICK_IMAGE_ALBUM_REQUEST = 2;            //Pick album photo code
    private GridLayoutManager gridLayout;                       //Photo album layout

    private final int REQUEST_PERMISSION_STORAGE_WRITE = 100;   //Storage write request permission code

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public EventDetailFragment() {
    }

    //ONCREATE METHOD
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (getArguments().containsKey(ARG_EVENT_TITLE)) {
            // Load the event content specified by the fragment
            // arguments.
            Activity activity = this.getActivity();

            //Get the event with the given title from the database
            SugarContext.init(activity);
            mItem = EventContent.getItem(getArguments().getString(ARG_EVENT_TITLE));
            SugarContext.terminate();

            //Set event title as the title of the toolbar
            getActivity().setTitle(mItem.getTitle());
        }
    }

    //ONCREATEVIEW METHOD
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.event_detail, container, false);

        //Get the elements from the view
        editTitle = (EditText)rootView.findViewById(R.id.event_title);
        editPlace = (EditText)rootView.findViewById(R.id.event_place);
        editDescription = (EditText)rootView.findViewById(R.id.event_description);
        datePickerInitial = (DatePicker) rootView.findViewById(R.id.initial_date);
        datePickerFinal = (DatePicker) rootView.findViewById(R.id.final_date);
        photo = (ImageView) rootView.findViewById(R.id.photo);
        photoDeleteButton = (ImageButton) rootView.findViewById(R.id.delete_button_image);
        photoText = (TextView) rootView.findViewById(R.id.add_photo_text);
        addPhotoAlbum = (ImageView) rootView.findViewById(R.id.add_photo_button);
        saveButton = (ImageButton)rootView.findViewById(R.id.save_button);
        deleteButton = (ImageButton)rootView.findViewById(R.id.delete_button);

        //Set the title, place and description inputs with the data obtained from DB
        editTitle.setText(mItem.getTitle());
        editPlace.setText(mItem.getPlace());
        editDescription.setText(mItem.getDescription());

        imageList = new ArrayList<String>();     //List of album images

        //Check if user has permissions to write into the storage
        if (ContextCompat.checkSelfPermission(getActivity(), Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(getActivity(), new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, REQUEST_PERMISSION_STORAGE_WRITE);
        }

        //Get main image path from the database
        mainImagePath = mItem.getUrlImage();

        //Set the main image of the view if there is one stored in the database event item
        try{
            if (mainImagePath.compareTo("") != 0) {
                Uri imageUri = Uri.parse(mItem.getUrlImage());

                Picasso.with(getActivity()).load(new File(imageUri.getPath())).centerCrop().fit().into(photo);
                photoDeleteButton.setVisibility(View.VISIBLE);
                photoText.setVisibility(View.INVISIBLE);
            }else{
                photoDeleteButton.setVisibility(View.INVISIBLE);
                photoText.setVisibility(View.VISIBLE);
            }
        } catch (Exception e) {
            Toast toast = Toast.makeText(this.getActivity(), R.string.toast_image_error, Toast.LENGTH_LONG);
            toast.show();
        }

        //Set click listener on the event main photo.
        //When clicked, calls the device gallery and let the user pick a photo
        photo.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v){
                Intent galleryIntent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                galleryIntent.setType("image/*");

                startActivityForResult(galleryIntent, PICK_IMAGE_REQUEST);
            }
        });

        //Set click listener on the event main photo delete button
        //Refresh the view of the main event photo
        photoDeleteButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v){
                photo.setImageResource(R.mipmap.ic_add);
                photoDeleteButton.setVisibility(View.INVISIBLE);
                photoText.setVisibility(View.VISIBLE);
                mainImagePath = "";
            }
        });

        //Get initial and final date of the event from the database
        String initialDate = mItem.getInitialDate();
        String finalDate = mItem.getFinalDate();

        //Set the dates of the datepickers with the received dates from the database
        try {
            Calendar calendarInitial = Calendar.getInstance();
            calendarInitial.setTime(dateFormat.parse(initialDate));
            datePickerInitial.updateDate(calendarInitial.get(Calendar.YEAR), calendarInitial.get(Calendar.MONTH), calendarInitial.get(Calendar.DAY_OF_MONTH));

            Calendar calendarFinal = Calendar.getInstance();
            calendarFinal.setTime(dateFormat.parse(finalDate));
            datePickerFinal.updateDate(calendarFinal.get(Calendar.YEAR), calendarFinal.get(Calendar.MONTH), calendarFinal.get(Calendar.DAY_OF_MONTH));
        }
        catch(Exception e) {
            //java.text.ParseException: Unparseable date: Getting error
            System.out.println("Excep"+e);
        }

        //Set click listener on the "Add new photo" option of the image album
        //When clicked, calls the device gallery and let the user pick a photo
        addPhotoAlbum.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v){
                Intent galleryIntent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                galleryIntent.setType("image/*");

                startActivityForResult(galleryIntent, PICK_IMAGE_ALBUM_REQUEST);
            }
        });

        //Get the list of image paths from the database, parse them to JSON and add them to the list
        try {
            JSONArray jsonList = new JSONArray(mItem.getUrlImageList());

            for (int i = 0; i < jsonList.length(); i++) {
                imageList.add(jsonList.getString(i));
            }

        } catch (JSONException e) {
            e.printStackTrace();
        }

        //Set the album layout with 3 columns
        gridLayout = new GridLayoutManager(getActivity(), 3);

        //Set the recycler view of the images album
        RecyclerView recyclerView = rootView.findViewById(R.id.image_list);
        assert recyclerView != null;
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(gridLayout);
        setupRecyclerView((RecyclerView) recyclerView);

        //Delete button click listener
        deleteButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {
                Context context = v.getContext();
                //Delete the given Event from the database
                SugarContext.init(context);
                int messageNumber = EventContent.delete(mItem);
                SugarContext.terminate();

                Toast toast = Toast.makeText(context, messageNumber, Toast.LENGTH_LONG);
                toast.show();

                //In twoPane mode, restart the current activity.
                //In single pane mode, close the current activity and goes to the previous one
                if (getActivity() instanceof EventDetailActivity){
                    getActivity().finish();
                }else if(getActivity() instanceof EventFutureListActivity){
                    Intent intent = new Intent(getActivity(), EventFutureListActivity.class);
                    startActivity(intent);
                    getActivity().finish();
                }else if(getActivity() instanceof  EventPastListActivity){
                    Intent intent = new Intent(getActivity(), EventPastListActivity.class);
                    startActivity(intent);
                    getActivity().finish();
                }else if(getActivity() instanceof  CalendarActivity){
                    Intent intent = new Intent(getActivity(), CalendarActivity.class);
                    startActivity(intent);
                    getActivity().finish();
                }
            }
        });

        //Save button click listener
        saveButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {
                Context context = v.getContext();
                //Create the dateStrings with the data obtained from datepickers
                String initialPickerDate = datePickerInitial.getDayOfMonth() + "/" + (1+datePickerInitial.getMonth()) + "/" + datePickerInitial.getYear();
                String finalPickerDate = datePickerFinal.getDayOfMonth() + "/" + (1+datePickerFinal.getMonth()) + "/" + datePickerFinal.getYear();
                boolean correctDates = false;

                //Parse the formed data strings and compare them
                try{
                    Calendar calendarInitial = Calendar.getInstance();
                    calendarInitial.setTime(dateFormat.parse(initialPickerDate));

                    Calendar calendarFinal = Calendar.getInstance();
                    calendarFinal.setTime(dateFormat.parse(finalPickerDate));

                    correctDates = calendarInitial.getTimeInMillis() <= calendarFinal.getTimeInMillis();
                }
                catch(Exception e) {
                    //java.text.ParseException: Unparseable date: Getting error
                    System.out.println("Excep"+e);
                }


                //Check if event title is empty
                if ((editTitle.getText().toString().trim().length() == 0)){
                    Toast toast = Toast.makeText(context, R.string.toast_title_empty, Toast.LENGTH_LONG);
                    toast.show();
                }else if (!correctDates){
                    //Check if the initial date is previous to the final date
                    Toast toast = Toast.makeText(context, R.string.toast_dates_error, Toast.LENGTH_LONG);
                    toast.show();
                }else{
                    //Create a new event item with the data obtained from the inputs
                    EventContent.EventItem newEvent = new EventContent.EventItem();
                    newEvent.setTitle(editTitle.getText().toString());
                    newEvent.setPlace(editPlace.getText().toString());
                    newEvent.setDescription(editDescription.getText().toString());
                    newEvent.setUrlImage(mainImagePath);
                    newEvent.setInitialDate(initialPickerDate);
                    newEvent.setFinalDate(finalPickerDate);

                    JSONArray jsonList = new JSONArray(imageList);
                    String arrayList = jsonList.toString();
                    newEvent.setUrlImageList(arrayList);

                    //Modify the event in the database with the new values
                    SugarContext.init(context);
                    int messageNumber = EventContent.modify(newEvent, mItem.getTitle());
                    SugarContext.terminate();

                    Toast toast = Toast.makeText(context, messageNumber, Toast.LENGTH_LONG);
                    toast.show();

                    //In twoPane mode, restart the current activity.
                    //In single pane mode, close the current activity and goes to the previous one
                    if (getActivity() instanceof EventDetailActivity){
                        getActivity().finish();
                    }else if(getActivity() instanceof EventFutureListActivity){
                        Intent intent = new Intent(getActivity(), EventFutureListActivity.class);
                        startActivity(intent);
                        getActivity().finish();
                    }else if(getActivity() instanceof  EventPastListActivity){
                        Intent intent = new Intent(getActivity(), EventPastListActivity.class);
                        startActivity(intent);
                        getActivity().finish();
                    }else if(getActivity() instanceof  CalendarActivity){
                        Intent intent = new Intent(getActivity(), CalendarActivity.class);
                        startActivity(intent);
                        getActivity().finish();
                    }
                }
            }
        });

        //Hide keyboard when clicking outside the input elements
        rootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                // Get the input method manager
                InputMethodManager inputMethodManager = (InputMethodManager)
                        view.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
                // Hide the soft keyboard
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(),0);
            }
        });
        return rootView;
    }

    //SETUPRECYCLERVIEW METHOD
    //Setup of the recycler View
    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new ImageListAdapter(this.getContext(), imageList);
        recyclerView.setAdapter(adapter);
    }

    //ONACTIVITYRESULT METHOD
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
            //Pick image from gallery action
            if (requestCode == PICK_IMAGE_REQUEST && resultCode == RESULT_OK && null != data) {
                try {
                    Uri selectedImage = data.getData();

                    //Get the real path of the file
                    String[] filePathColumn = {MediaStore.Images.Media.DATA};
                    Cursor cursor = getActivity().getContentResolver().query(selectedImage, filePathColumn, null, null, null);
                    cursor.moveToFirst();
                    int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
                    mainImagePath = cursor.getString(columnIndex);
                    cursor.close();

                    //Set the imageview with the obtained path from the image gallery intent
                    Picasso.with(getActivity()).load(selectedImage).centerCrop().fit().into(photo);
                    photoDeleteButton.setVisibility(View.VISIBLE);
                    photoText.setVisibility(View.INVISIBLE);
                } catch (Exception e) {
                    Toast toast = Toast.makeText(this.getActivity(), R.string.toast_image_error, Toast.LENGTH_LONG);
                    toast.show();
                }
            }

        //Pick image from gallery action
        if (requestCode == PICK_IMAGE_ALBUM_REQUEST && resultCode == RESULT_OK && null != data) {
            try {
                Uri selectedImage = data.getData();

                //Get the real path of the file
                String[] filePathColumn = {MediaStore.Images.Media.DATA};
                Cursor cursor = getActivity().getContentResolver().query(selectedImage, filePathColumn, null, null, null);
                cursor.moveToFirst();
                int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
                String albumImagePath = cursor.getString(columnIndex);
                cursor.close();

                //Add the image path to the list and update the recycler view
                imageList.add(albumImagePath);
                adapter.notifyDataSetChanged();

            } catch (Exception e) {
                Toast toast = Toast.makeText(this.getActivity(), R.string.toast_image_error, Toast.LENGTH_LONG);
                toast.show();
            }
        }
    }

}
