package edu.uoc.android.memoriesplanner;

import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.Toast;

import com.facebook.FacebookSdk;
import com.facebook.share.model.SharePhoto;
import com.facebook.share.model.SharePhotoContent;
import com.facebook.share.widget.ShareDialog;
import com.squareup.picasso.Picasso;

import java.io.File;

/**
 * An activity representing the clicked image of the album.
 * It contains the option to share the image with Whatsapp or Facebook
 */

public class ImageActivity extends AppCompatActivity {

    //Intent arguments
    public static final String ARG_IMAGE_URL = "image_url";
    public static final String ARG_IMAGE_POS = "image_pos";

    private ImageView image;                    //Image container
    private ImageButton photoDeleteButton;      //Delete image button
    private String imageUrl;                    //Path of the image
    private int position;                       //Position of the image in the view

    private ShareDialog shareDialog;            //Facebook sharing photo dialog

    //ONCREATE METHOD
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_image);
        FacebookSdk.sdkInitialize(this.getApplicationContext());

        //Top toolbar
        Toolbar toolbar = (Toolbar) findViewById(R.id.image_toolbar);
        toolbar.setTitle("");
        setSupportActionBar(toolbar);

        // Show the Back button in the action bar.
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        //Get the view elements
        image = (ImageView) findViewById(R.id.image);
        photoDeleteButton = (ImageButton) findViewById(R.id.delete_button_image);

        //Get the intent arguments: image path and image position
        Intent intent = getIntent();
        imageUrl = intent.getStringExtra(ARG_IMAGE_URL);
        position = intent.getIntExtra(ARG_IMAGE_POS, -1);

        //Set the imageView with the image of the taken path
        Uri uri = Uri.parse(imageUrl);
        Picasso.with(this).load(new File(uri.getPath())).centerCrop().fit().into(image);
    }

    //ONCREATEOPTIONSMENU METHOD.
    //Set the top bar menu
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.image_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    //ONOPTIONSITEMSELECTED METHOD
    //Navigation between activities using the toolbar menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent navigation_intent;

        switch (item.getItemId()) {
            case R.id.whatsapp:
                Uri imageUri = Uri.parse(imageUrl);
                Intent shareIntent = new Intent();
                shareIntent.setAction(Intent.ACTION_SEND);
                //Target whatsapp
                shareIntent.setPackage("com.whatsapp");
                //Add the image path to the intent
                shareIntent.putExtra(Intent.EXTRA_STREAM, imageUri);
                shareIntent.setType("image/*");
                shareIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);

                try {
                    startActivity(shareIntent);
                } catch (android.content.ActivityNotFoundException ex) {
                    //Whatsapp not found
                    Toast toast = Toast.makeText(this, R.string.toast_no_whatsapp, Toast.LENGTH_LONG);
                    toast.show();
                }
                break;
            case R.id.facebook:
                shareDialog = new ShareDialog(this);

                try{
                    //Check if Facebook is installed
                    ApplicationInfo info = getPackageManager().
                            getApplicationInfo("com.facebook.katana", 0 );

                    //Share photo with Facebook
                    SharePhoto image = new SharePhoto.Builder().setBitmap(BitmapFactory.decodeFile(imageUrl)).build();
                    if (ShareDialog.canShow(SharePhotoContent.class)) {
                        SharePhotoContent content = new SharePhotoContent.Builder().addPhoto(image).build();
                        shareDialog.show(content);
                    }
                } catch( PackageManager.NameNotFoundException e ){
                    //Facebook not found
                    Toast toast = Toast.makeText(this, R.string.toast_no_facebook, Toast.LENGTH_LONG);
                    toast.show();
                }
                break;
            case R.id.future_events:
                navigation_intent = new Intent(this, EventFutureListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.past_events:
                navigation_intent = new Intent(this, EventPastListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.calendar:
                navigation_intent = new Intent(this, CalendarActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case android.R.id.home:
                this.finish();
                break;
            default:
                break;
        }
        return true;
    }
}
