package edu.uoc.android.memoriesplanner.adapter;

import android.content.Context;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.PopupMenu;

import com.squareup.picasso.Picasso;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.memoriesplanner.ImageActivity;
import edu.uoc.android.memoriesplanner.R;

/**
 * Adapter class of the recycler view that contains the list of images
 */
public class ImageListAdapter extends RecyclerView.Adapter<ImageListAdapter.ViewHolder> {

    private Context context;
    private List<String> mImagesUrl;
    private int imagePos;
    private String imageUrl;

    //When a image from the list is clicked, display a popup menu
    //with the options: "View image" and "Delete image"
    private final View.OnClickListener mOnClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View view) {
            //Get the image path and position from the view tag
            List<String> arrayTag = (List<String>) view.getTag();
            context = view.getContext();
            imageUrl = (String) arrayTag.get(0);
            imagePos =  Integer.parseInt(arrayTag.get(1));

            //Create and inflate the popup menu
            PopupMenu menu = new PopupMenu(context, view);
            menu.inflate(R.menu.album_photo_menu);

            //Menu on click listener
            menu.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
                @Override
                public boolean onMenuItemClick(MenuItem menuItem) {
                    switch (menuItem.getItemId()) {
                        case R.id.view_photo:
                            //Open the ImageActivity which display the full-size photo
                            Intent intent = new Intent(context, ImageActivity.class);
                            intent.putExtra(ImageActivity.ARG_IMAGE_URL, imageUrl);
                            intent.putExtra(ImageActivity.ARG_IMAGE_POS, imagePos);

                            context.startActivity(intent);
                            break;
                        case R.id.delete_photo:
                            //Remove the image with the given position from the view
                            mImagesUrl.remove(mImagesUrl.get(imagePos));
                            notifyDataSetChanged();
                            break;
                        default:
                            break;
                    }
                    return true;
                }
            });
            menu.show();
        }
    };


    //Builder
    public ImageListAdapter(Context context, List<String> items) {
        mImagesUrl = items;
        this.context = context;
    }

    //Inflate the image items into the recycler view
    @Override
    public ImageListAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.image_list_content, parent, false);
        return new ImageListAdapter.ViewHolder(view);
    }

    //Set the image holder with the image path taken from the list
    @Override
    public void onBindViewHolder(final ImageListAdapter.ViewHolder holder, int position) {
        Picasso.with(context).load(new File(mImagesUrl.get(position))).centerCrop().fit().into(holder.mImage);
        //holder.mImage.setImageBitmap(BitmapFactory.decodeFile(mImagesUrl.get(position)));

        //Add the image path and position to the tag
        List<String> arrayTag = new ArrayList<>();
        arrayTag.add(mImagesUrl.get(position));
        arrayTag.add(position+"");

        holder.itemView.setTag(arrayTag);
        holder.itemView.setOnClickListener(mOnClickListener);
    }

    //Get the total number of images in the list
    @Override
    public int getItemCount() {
        return mImagesUrl.size();
    }


    //Viewholder class representing a single album image into the recycler view
    class ViewHolder extends RecyclerView.ViewHolder {
        final ImageView mImage;

        ViewHolder(View view) {
            super(view);
            mImage = (ImageView) view.findViewById(R.id.image);
        }
    }
}
