package edu.uoc.android.memoriesplanner.adapter;

import android.app.Activity;
import android.app.Fragment;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import java.util.List;

import edu.uoc.android.memoriesplanner.EventDetailActivity;
import edu.uoc.android.memoriesplanner.EventDetailFragment;
import edu.uoc.android.memoriesplanner.NewEventActivity;
import edu.uoc.android.memoriesplanner.R;
import edu.uoc.android.memoriesplanner.model.EventContent;

/**
 * Adapter class of the recycler view that contains the list of events
 */

public class ListAdapter extends RecyclerView.Adapter<ListAdapter.ViewHolder> {

    private final Activity mParentActivity;
    private List<EventContent.EventItem> mValues;
    private final boolean mTwoPane;

    //Set the Onclick listener to the Event item.
    //In twoPane mode, display the EventDetailFragment into the second panel of the screen
    //In single panel mode, open the activity EventDetailActivity
    private final View.OnClickListener mOnClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View view) {
            EventContent.EventItem item = (EventContent.EventItem) view.getTag();
            if (mTwoPane) {
                Bundle arguments = new Bundle();
                arguments.putString(EventDetailFragment.ARG_EVENT_TITLE, item.getTitle());
                Fragment fragment = new Fragment();
                fragment.setArguments(arguments);
                mParentActivity.getFragmentManager().beginTransaction()
                        .replace(R.id.event_detail_container, fragment)
                        .commit();
            } else {
                Context context = view.getContext();
                Intent intent = new Intent(context, EventDetailActivity.class);
                intent.putExtra(EventDetailFragment.ARG_EVENT_TITLE, item.getTitle());

                context.startActivity(intent);
            }
        }
    };


    //Builder
    public ListAdapter(Activity parent, List<EventContent.EventItem> items, boolean twoPane) {
        mValues = items;
        mParentActivity = parent;
        mTwoPane = twoPane;
    }

    //Inflate the event items into the recycler view
    @Override
    public ListAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.event_list_content, parent, false);
        return new ListAdapter.ViewHolder(view);
    }

    //Set the event holder with the attributes of the event taken from the list
    @Override
    public void onBindViewHolder(final ListAdapter.ViewHolder holder, int position) {
        holder.mTitle.setText(mValues.get(position).getTitle());
        holder.mStartDate.setText(mValues.get(position).getInitialDate());
        holder.mEndDate.setText(mValues.get(position).getFinalDate());

        holder.itemView.setTag(mValues.get(position));
        holder.itemView.setOnClickListener(mOnClickListener);
    }

    //Get the total number of events in the list
    @Override
    public int getItemCount() {
        return mValues.size();
    }


    //Viewholder class representing a single event into the recycler view list
    class ViewHolder extends RecyclerView.ViewHolder {
        final TextView mTitle;
        final TextView mStartDate;
        final TextView mEndDate;

        ViewHolder(View view) {
            super(view);
            mTitle = (TextView) view.findViewById(R.id.title);
            mStartDate = (TextView) view.findViewById(R.id.start_date);
            mEndDate = (TextView) view.findViewById(R.id.end_date);
        }
    }
}
