package edu.uoc.android.memoriesplanner.model;

import com.orm.SugarRecord;

import java.util.List;

import edu.uoc.android.memoriesplanner.R;


/**This class represents the Event Objects for the app
 * and the database interaction
 *
* */
public class EventContent {

    //GETEVENTS METHOD
    //Retrieve all the events from the database
    public static List<EventItem> getEvents(){
        return EventItem.listAll(EventItem.class);
    }

    //GETITEM METHOD
    //Retrieve an event with a given Title from the database
    public static EventItem getItem(String title){
        List<EventItem> eventItemList = EventItem.find(EventItem.class, "title = ?", title);
        return eventItemList.get(0);
    }

    //EXISTS METHOD
    //Check if the event with the given title already exists in the database
    public static boolean exists(String title) {
        List<EventItem> eventItemList = EventItem.find(EventItem.class, "title = ?", title);
        if (eventItemList == null || eventItemList.size() == 0) {
            return false;
        } else {
            return true;
        }
    }

    //MODIFY METHOD
    //Search for the given event into the database and update all its attributes if it exists
    public static int modify(EventItem event, String lastTitle){
        List<EventItem> eventItemList = EventItem.find(EventItem.class, "title = ?", lastTitle);
        if (eventItemList == null || eventItemList.size() == 0) {
            return R.string.toast_text_not_found_event;
        } else {
            EventItem updatedEvent = eventItemList.get(0);
            updatedEvent.setTitle(event.getTitle());
            updatedEvent.setPlace(event.getPlace());
            updatedEvent.setDescription(event.getDescription());
            updatedEvent.setInitialDate(event.getInitialDate());
            updatedEvent.setFinalDate(event.getFinalDate());
            updatedEvent.setUrlImage(event.getUrlImage());
            updatedEvent.setUrlImageList(event.getUrlImageList());

            updatedEvent.save();
            return R.string.toast_text_save_event;
        }
    }

    //DELETE METHOD
    //Search for the given event into the database and delete it if exists
    public static int delete(EventItem event){
        List<EventItem> eventItemList = EventItem.find(EventItem.class, "title = ?", event.getTitle());
        if (eventItemList == null || eventItemList.size() == 0) {
            return R.string.toast_text_not_found_event;
        } else {
            eventItemList.get(0).delete();
            return R.string.toast_text_delete_event;
        }
    }

    //DELETEALL METHOD
    //Delete every event from the database
    public static void deleteAll(){
        EventItem.deleteAll(EventItem.class);
    }

    /**
     * Class of the item that represents an Event
     */
    public static class EventItem extends SugarRecord {
        private String title;
        private String description;
        private String place;
        private String initialDate;
        private String finalDate;
        private String urlImage;
        private String urlImageList;

        //Empty builder
        public EventItem(){}

        //Builder of an EventItem object
        public EventItem(String title, String description, String place, String initialDate, String finalDate, String urlImage, String urlImageList){
            this.title = title;
            this.description = description;
            this.place = place;
            this.initialDate = initialDate;
            this.finalDate = finalDate;
            this.urlImage = urlImage;
            this.urlImageList = urlImageList;
        }

        //TOSTRING METHOD
        //Returns the title of the Event
        @Override
        public String toString(){return this.title;}

        //Getters and setters of the Event Object attributes
        public String getTitle(){return this.title;}

        public void setTitle(String title){this.title = title;}

        public String getDescription(){return this.description;}

        public void setDescription(String description){this.description = description;}

        public String getPlace(){return this.place;}

        public void setPlace(String place){this.place = place;}

        public String getInitialDate(){return this.initialDate;}

        public void setInitialDate(String initialDate){this.initialDate = initialDate;}

        public String getFinalDate(){return this.finalDate;}

        public void setFinalDate(String finalDate){this.finalDate = finalDate;}

        public String getUrlImage(){return this.urlImage;}

        public void setUrlImage(String urlImage){this.urlImage = urlImage;}

        public String getUrlImageList(){return this.urlImageList;}

        public void setUrlImageList(String urlImageList){this.urlImageList = urlImageList;}
    }
}
